from typing import Tuple, TypedDict

__version__: str

class DecodeError(ValueError): ...
class EncodeError(ValueError): ...
class ParamError(SyntaxError): ...

class BBox(TypedDict):
    e: float
    n: float
    s: float
    w: float

class Neighbors(TypedDict):
    e: str
    n: str
    ne: str
    nw: str
    s: str
    se: str
    sw: str
    w: str

def bbox(hash: str) -> BBox: ...
def decode_exact(hash: str) -> Tuple[float, float, float, float]: ...
def decode(hash: str) -> Tuple[float, float]: ...
def encode(lat: float, lon: float, len: int = 12) -> str: ...
def neighbor(hash: str, direction: str) -> str: ...
def neighbors(hash: str) -> Neighbors: ...



from contextlib import nullcontext

import pytest

from geohashr import EncodeError, ParamError, neighbor, neighbors


@pytest.mark.parametrize(
    ("hash", "raises", "out"),
    [
        ("000000000000", pytest.raises(EncodeError), {}),
        ("zzzzzzzzzzzz", pytest.raises(EncodeError), {}),
        (
            "bgr96qxvpd46",
            nullcontext(),
            {
                "e": "bgr96qxvpd4d",
                "n": "bgr96qxvpd47",
                "ne": "bgr96qxvpd4e",
                "nw": "bgr96qxvpd45",
                "s": "bgr96qxvpd43",
                "se": "bgr96qxvpd49",
                "sw": "bgr96qxvpd41",
                "w": "bgr96qxvpd44",
            },
        ),
    ],
)
def test_neighbors(hash, raises, out):
    with raises:
        assert out == neighbors(hash)


@pytest.mark.parametrize(
    ("direction", "out"),
    [
        ("e", "bgr96qxvpd4d"),
        ("n", "bgr96qxvpd47"),
        ("ne", "bgr96qxvpd4e"),
        ("nw", "bgr96qxvpd45"),
        ("s", "bgr96qxvpd43"),
        ("se", "bgr96qxvpd49"),
        ("sw", "bgr96qxvpd41"),
        ("w", "bgr96qxvpd44"),
    ],
)
def test_neighbor(direction, out):
    assert neighbor("bgr96qxvpd46", direction) == out


def test_neighbor_invalid_direction():
    with pytest.raises(ParamError):
        assert neighbor("bgr96qxvpd46", "g")



import pytest

from geohashr import decode, encode


@pytest.mark.parametrize("hash", ["000000000000", "zzzzzzzzzzzz", "bgr96qxvpd46"])
def test_cycle(hash):
    assert hash == encode(*decode(hash))


def test_simple_encode():
    assert encode(57.64911, 10.40744) == "u4pruydqqvj8"
